# *************************************************************
#
# gpg.tcl --
#
# This file contains procs, that deal with the 
# multialgorithmic nature of GPG
#
# Version: v3-gpg1.0 (alpha)
#
# Thu Aug 13 03:59:31 MEST 1998 -- Markus.Gruber@ilo.baynet.de
#
#
###########################################
#
# NOTE:
# * * * GPG IS ALPHA SOFTWARE * * *
# Use it at your own risk
# No guarantee, that the stuff in this file
# fits to future versions of GPG
#
###########################################
#
# *************************************************************


# Init stuff
#
proc Gpg_Init { } {
    global gpg

    if {$gpg(enabled)} {
	Preferences_Add "GPG interface" {}\
	 [list \
	 [list gpg(modulepath) gpgModulePath {/usr/local/lib/gnupg} "GPG Modules Path"\
"GPG is able to dynamically load cipher, digest, pubkey etc.
extension algorithms.
Specify the directory of extension modules you want to use."] ]
	#
	set ciphermods {twofish skipjack idea}
	set digestmods {tiger}
	set pubkeymods {rsa}
	#
	
	# Hm, look what extensions there are
	catch { cd $gpg(modulepath) }

	# Initialize
	set gpg(ciphermods) {}
	set gpg(digestmods) {}
	set gpg(pubkeymods) {}

	foreach file [glob -nocomplain *] {
	    if { ![file isdirectory $file] } {
		if {[lsearch $ciphermods $file] >= 0} {
		    lappend gpg(ciphermods) $file
		} elseif {[lsearch $digestmods $file] >= 0} {
		    lappend gpg(digestmods) $file
		} elseif {[lsearch $pubkeymods $file] >= 0} {
		    lappend gpg(pubkeymods) $file
		}
	    }
	}

	set gpg(algomods) [concat $gpg(ciphermods) \
				$gpg(digestmods) $gpg(pubkeymods)]
	
	set gpg(cipheralgos) [concat {cast5 blowfish blowfish160} $gpg(ciphermods)]
	set gpg(digestalgos) [concat {sha1 md5 ripemd160} $gpg(digestmods)]
	set gpg(compressalgos) {zip zlib none}
	#
	Preferences_Add "GPG interface" {}\
	 [list \
	 [list gpg(cipheralgo)  gpgCipherAlgo [concat CHOICE $gpg(cipheralgos)] \
		"Default Cipher Algo"\
"NOTE: If you mark twofish, skipjack or idea as your default cipher, please 
make sure, that you have these extension modules installed."]\
	 [list gpg(digestalgo)  gpgDigestAlgo [concat CHOICE $gpg(digestalgos)] \
		"Default Digest Algo"\
"NOTE: If you mark tiger as your default digest, please 
make sure, that the tiger extension module is installed."]\
	 [list gpg(compressalgo)  gpgCompressAlgo [concat CHOICE $gpg(compressalgos)] "Default Compress Algo"\
"The algorithm, gpg uses to compress the text before encrypting.
You have the choice between the zlib and the zip algo.
If you choose none, the text is left uncompressed."]]
    }
}

#
# Simple dialog to change default cipher, digest ... algorithms
# 
proc Gpg_ChooseAlgos {} {
    global gpg
    set t [toplevel .gregor]
    wm title $t "Default Algorithms"
    wm resizable $t 0 0
    set m1 [frame $t.main1]
    pack $m1 -side left
    set m2 [frame $t.main2]
    pack $m2 -side left
    #
    # Cipheralgo
    set f [frame $m2.frame1]
    pack $f -side top -expand 1 -fill x
    set l [label $m1.cipher -text "Default cipheralgo"]
    pack $l -side top
    foreach algo $gpg(cipheralgos) {
        set r [radiobutton $f.$algo -variable gpg(cipheralgo) \
						-text $algo -value $algo]
	pack $r -side left
    }
    #
    # Digestalgo
    set f [frame $m2.frame2]
    pack $f -side top -expand 1 -fill x
    set l [label $m1.digest -text "Default digestalgo"]
    pack $l -side top
    foreach algo $gpg(digestalgos) {
        set r [radiobutton $f.$algo -variable gpg(digestalgo) \
						-text $algo -value $algo]
	pack $r -side left
    }
    #
    # Compressalgo
    set f [frame $m2.frame3]
    pack $f -side top -expand 1 -fill x
    set l [label $m1.compress -text "Default compressalgo"]
    pack $l -side top
    foreach algo $gpg(compressalgos) {
        set r [radiobutton $f.$algo -variable gpg(compressalgo) \
						-text $algo -value $algo]
	pack $r -side left
    }
    #
    # OK
    set b [button $f.ok -text "OK" -command "destroy $t"]
    pack $b -side right
}

#
# building the gpg commandline
#
proc Gpg_Arglist {arglistvar {password {}}} {
    global gpg
    upvar $arglistvar arglist

    foreach mod $gpg(algomods) {
	set arglist [concat [list \
		--load-extension $mod] $arglist]
    }
    #
    # cipheralgo
    if {[lsearch $gpg(cipheralgos) $gpg(cipheralgo)] >= 0} {
       set arglist [concat [list \
		--cipher-algo $gpg(cipheralgo)] $arglist]
    }
    #
    # digestalgo
    if {[lsearch $gpg(digestalgos) $gpg(digestalgo)] >= 0} {
        set arglist [concat [list \
		--digest-algo $gpg(digestalgo)] $arglist]
    }
    #
    # compressalgo
    switch $gpg(compressalgo) {
	zip  { set compressid 1 }
	zlib { set compressid 2 }
    }
    if [info exists compressid] {
	set arglist [concat [list \
		--compress-algo $compressid] $arglist]
    } else {
	set arglist [concat [list \
		-z 0] $arglist]
    }
    if {!($password == {})} {
	set arglist [concat [list --passphrase-fd 0] $arglist]
    }
}
