# pgpInterface.tcl -- 
# created by monnier@didec26.epfl.ch on Mon Dec 12 17:34:38 1994

# 
# 
# 

# $Log: pgpExec.tcl,v $
# Revision 1.11  1998/01/22  00:45:06  bwelch
#     Hack to use aixterm for PGP.
#
# Revision 1.10  1997/12/22  20:52:00  bwelch
# file delete
#
# Revision 1.9  1997/07/25  17:13:23  bwelch
# Fixed pattern match to handle PGP 5.0 date format.
#
# Revision 1.8  1997/07/12  23:05:12  bwelch
#     Fixed PGP key extraction from the web servers.
#     Fixed handling of failed signatures so you still see the message.
#
# Revision 1.7  1997/06/03  18:29:55  bwelch
# Added PGP grab-focus and use-expecttk options.
# Removed +keepbinary=off flag from PGP uses.
# PGP bin directory is added to the front of PATH, if necessary
#
# Revision 1.6  1997/01/25  05:29:23  bwelch
#     Tweaked PgpExec_KeyList that returns a list of keys.
#     Tweaked patterns on PGP output.
#     Added Pgp_ShortenOutput
#
# Revision 1.5  1996/12/21  00:57:12  bwelch
# Log errors from PGP key extraction
#
# Revision 1.4  1996/12/01  20:13:59  bwelch
# Added Pgp_InterpretOutput
# Added timeouts on password caching.
#
# Revision 1.3  1996/03/22  18:42:54  bwelch
# Added Mh_Rename
# .
#
# Revision 1.2  1995/05/24  05:58:04  bwelch
# Updates from Stefan
#
# Revision 1.1  1995/05/19  17:36:16  bwelch
# Initial revision
#
# Revision 1.2  1995/03/22  19:14:21  welch
# More new code from Stefan
#
# Revision 1.1  1994/12/30  21:49:00  welch
# Initial revision
#
# Revision 1.1  1994/12/17  20:19:16  monnier
# Initial revision
#

# execs pgp with the usual flags
# added pgp5 support
# added gpg support
proc PgpExec { version arglist outvar {privatekey {}} {interactive 0} } {
    upvar $outvar output
    global pgp pgp5 gpg env

    Exmh_Debug "PgpExec $version $arglist $outvar $privatekey $interactive"
    
    if {! [set ${version}(enabled)]} {
	error "<[string toupper $version]> isn't enabled"
    }
    set output {}

    if {![set ${version}(keeppass)]} {
	Pgp_ClearPassword $version
    }

    if {$interactive || !([set ${version}(keeppass)] || ($privatekey == {}))} {
	return [PgpExec_Interactive $version $arglist output]
    } else {
	if {$privatekey == {}} {
	    return [PgpExec_Batch $version $arglist output]
	} else {
	    set p [Pgp_GetPass $version $privatekey]
	    if {[string length $p] == 0} {
		return 0
	    }
	    return [PgpExec_Batch $version $arglist output $p]
	}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_Batch { version arglist outvar {password {}} } {
    upvar $outvar output
    global pgp pgp5 gpg env

    switch $version {
	pgp  {	set tclcmd [concat \
	    	[list exec pgp +armorlines=0 +batchmode=on +verbose=0 +pager=cat] \
	    	$arglist] }
	pgp5 {	set tclcmd [concat \
		[list exec [lindex $arglist 0] +batchmode=on +verbose=0] \
		[lrange $arglist 1 end]] }
	gpg  {  Gpg_Arglist arglist $password
		set tclcmd [concat \
		[list exec gpg --batch] $arglist] }
    }

    if {$password == {}} {
	catch { unset env(PGPPASSFD) }
    } else {
	lappend tclcmd << $password
	if { ($version == "pgp") || ($version == "pgp5") } {
	    set env(PGPPASSFD) 0
	}
    }
	
    Exmh_Debug $tclcmd
    Exmh_Debug "Error status: [set result [catch {eval $tclcmd} output]]"
    regsub -all "\x07" $output "" output

    catch { unset env(PGPPASSFD) }

    return $result
}

# added pgp5 support
# added gpg support
proc PgpExec_Interactive { version arglist outvar } {
    global tcl_platform
    upvar $outvar output
    
    switch $version {
	pgp  { set pgpcmd pgp; set args [concat [list +armorlines=0] $arglist] }
	pgp5 { set pgpcmd [lindex $arglist 0]; set args [lrange $arglist 1 end] }
	gpg  { set pgpcmd gpg; set args $arglist; Gpg_Arglist arglist }
    }
    set shcmd "unset PGPPASSFD;
        $pgpcmd \"[join [Misc_Map x {
	    regsub {([$"\`])} $x {\\1} x
	    set dummy $x
        } $args] {" "}]\";
	echo
	echo press Return...;
        read dummy"

    set logfile [Mime_TempFile "xterm"]
    if { ( $tcl_platform(os) == "AIX" ) && [ file executable "/usr/bin/X11/aixterm" ] } {
        set xterm "aixterm"
    } else {
        set xterm "xterm"
    }
    #
    # Hint: XFree86 xterm does not support output logging (Markus)
    # -l and -lf not supported
    #
    set tclcmd {exec $xterm -l -lf $logfile -title PGP -e sh -c $shcmd}
    Exmh_Debug $tclcmd
    set result [catch $tclcmd]
    if [catch {open $logfile r} log] {
	set output {}
    } else {
	set output [read $log]
	close $log
    }

    # clean up the output
    regsub -all "\[\x0d\x07]" $output {} output
    regsub "^.*\nEnter pass phrase:" $output {} output
    regsub "\nPlaintext filename:.*" $output {} output
    regsub "^.*Just a moment\\.\\.+" $output {} output
    regsub "^.*Public key is required \[^\n]*\n" $output {} output
    set output [string trim $output]

    return $result
}

# added pgp5 support
# added gpg support
proc PgpExec_CheckPassword { version password key } {

    set tmpfile [Mime_TempFile "pwdin"]
    set outfile [Mime_TempFile "pwdout"]

    set out [open $tmpfile w 0600]
    puts $out "salut"
    close $out

    switch $version {
	pgp  { set arglist [list -as $tmpfile -o $outfile -u [lindex $key 0]] }
	pgp5 { set arglist [list pgps -bat $tmpfile -o $outfile -u [lindex $key 0]] }
	gpg  { set arglist [list -as -o $outfile -u [lindex $key 0] $tmpfile] }
    }

    PgpExec_Batch $version $arglist err $password
    File_Delete $tmpfile

    # pgp thinks he knows better how to name files !
    if {![file exists $outfile] && [file exists "$outfile.asc"]} {
	Mh_Rename "$outfile.asc" $outfile
    }

    switch $version {
	pgp  { set pat "PGP" ; set err_pat {(Error:[^\.]*)\.} }
	pgp5 { set pat {signature|Cannot} ; set err_pat "(Cannot\[^\n]*)\n" }
	gpg  { set pat "gpg" ; set err_pat "(signing failed:\[^\n]+)\n" }
    }
    if {![file exists $outfile]} {
	if {![regexp $pat $err]} {
	    # Probably cannot find pgp to execute.
    	    Exmh_Status !${err}!
	    error "<PGP> can't find $version"
	} else {
	    if [regexp $err_pat $err x match] {
	    	Exmh_Status ?${match}?
	    }
	    ExmhLog "<Pgp_GetPass> $err"
	}
	return 0
    } else {
	File_Delete $outfile
	return 1
    }
}
	
# wrapper for 'pgp -kv $pattern $keyring'
# returns a list of keys. Each "key" is a list whose first element is the keyID
# and the next ones are the corresponding userids
#
# added pgp5 support
# added gpg support
proc PgpExec_KeyList { version pattern keyring } {

    set pattern [string trimleft $pattern "<>|2"]
    switch $version {
	pgp	{ PgpExec_Batch pgp [list -kv $pattern $keyring] keylist }
	pgp5	{ set keylist [exec pgpk -l $pattern] }
	gpg	{ switch -regexp -- $keyring {
		      pub { set gpgaction --list-keys }
		      sec { set gpgaction --list-secret-keys }
		  }
		  if {$pattern == {}} {
		  	PgpExec_Batch gpg [list $gpgaction] keylist
		  } else {
			PgpExec_Batch gpg [list $gpgaction $pattern] keylist
		  }
	        }
    }
    #
    # drop revoked and noninteresting keys
    switch $version {
	pgp  {
	    regsub -all "\n(pub|sec) \[^\n]+\\*\\*\\* KEY REVOKED \\*\\*\\*(\n\[\t ]\[^\n]+)+" $keylist "" keylist }
	pgp5 { 
	    regsub -all "\n(pub|sec)\[\\?!\\*]? \[^\n]+\\*REVOKED\\*\[^\n]+(\n(sub) \[^\n]+)?(\n(uid) \[^\n]+)+" $keylist "" keylist
	    regsub -all "\n(pub@|pub%|sec@|ret) \[^\n]+(\n(sub) \[^n]+)?(\n(uid) \[^n]+)+" $keylist "" keylist }
	gpg  {
	    regsub -all "\n(sub|ssb|uid) \[^\n\]+" $keylist "" keylist }
    }
    #
    # Match out interesting stuff
    switch $version {
	pgp  { set keypattern {.*(pub|sec) +[0-9]+(/| +)([0-9A-F]+) +[0-9]+/ ?[0-9]+/ ?[0-9]+ +(.*)} }
	pgp5 { set keypattern ".*(pub|sec)\[\\?!\\*\\+]? +\[0-9]+ +0x\[0-9A-F]+ +\[0-9]+-\[0-9]+-\[0-9]+ +\[^\n]+(\nsub\[^\n]+)?(\nuid\[^\n]+)+" }
	gpg  { set keypattern {.*(pub|sec) +[0-9]+[A-Za-z]+/([0-9A-F]+) +[0-9]+-[0-9]+-[0-9]+ +(.*)} }
    }
    if { ![regexp $keypattern $keylist]} {
	return {}
    } else {
	Exmh_Debug $keylist
	switch $version {
	   pgp  { set keylist [split $keylist "\n"] }
	   pgp5 { regsub -all "\n\n" $keylist \x81 keylist; set keylist [split $keylist "\x81"] }
	   gpg  { set keylist [split $keylist "\n"] }
	}
	set keys {}
	set key {}

	# decide, wether to select private or public keys for pgp5
	switch -regexp -- $keyring {
	    pub { set type (pub|sec) }
	    sec { set type (sec) }
	}
	switch $version {
	    pgp  { set keypattern {^.*(pub|sec) +[0-9]+(/| +)([0-9A-F]+) +[0-9]+/ ?[0-9]+/[0-9]+ +(.*)$} }
	    pgp5 { set keypattern "^.*${type}\[\\?!\\*\\+]? +\[0-9]+ +0x(\[0-9A-F]+) +\[0-9]+-\[0-9]+-\[0-9]+ +\[^\n]+(\nsub\[^\n]+)?\nuid +(\[^\n]+)" }
	    gpg  { set keypattern {^.*(pub|sec) +[0-9]+[A-Za-z]+/([0-9A-F]+) +[0-9]+-[0-9]+-[0-9]+ +(.*)$} }
	}
	switch $version {
	    pgp  { set cmd { regexp $keypattern $line {} {} {} keyid userid } }
	    pgp5 { set cmd { regexp $keypattern $line {} {} keyid {} userid } }
	    gpg  { set cmd { regexp $keypattern $line {} {} keyid userid } }
	}
	foreach line $keylist {
            if [ eval $cmd ] {
		set key [list "0x$keyid" [string trim $userid]]
		lappend keys $key
	    }
	}
	return $keys
    }
}

# added pgp5 support
# added gpg support
proc PgpSetPath {} {
    global pgp pgp5 gpg env
    foreach version { pgp pgp5 gpg } {
    	if {[info exists ${version}(path)] && \
	    ([string length [string trim [set ${version}(path)]]] > 0) && \
	    ([lsearch -exact [split $env(PATH) :] [set ${version}(path)]] < 0)} {
		set env(PATH) [set ${version}(path)]:$env(PATH)
	}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_Init { version } {
    global pgp pgp5 gpg pgpConfig pgp5Config gpgConfig pgpPass pgp5Pass gpgPass env

    PgpSetPath

    if {![info exists env(LOCALHOST)]} {
	if [catch {exec uname -n} env(LOCALHOST)] {
	    set env(LOCALHOST) localhost
	}
    }

    set ${version}Pass() {}

    # Parse PGP config file (Dont parse gpg options file)
    switch $version {
	pgp  { set config pgp(pgppath)/config.txt }
	pgp5 { set config pgp(pgppath)/pgp.cfg }
    }
    if {[regexp pgp $version]} {
        PgpExec_ParseConfigTxt $config ${version}Config
    }
    
    switch $version {
	pgp  { set secring $pgp(pgppath)/secring.pgp }
	pgp5 { set secring $pgp(pgppath)/secring.skr }
	gpg  { set secring $gpg(gpgpath)/secring.gpg }
    }
    if {![file exists $secring]} {
	set ${version}(secring) {}
    } else {
	set ${version}(secring) $secring
    }

    set ${version}(privatekeys) [PgpExec_KeyList $version "" [set ${version}(secring)]]
    
    if [info exists ${version}Config(myname)] {
	set myname [string tolower [set ${version}Config(myname)]]
	foreach key [set ${version}(privatekeys)] {
	    if {[string first $myname [string tolower $key]] >= 0} {
		set ${version}(myname) $key
		break
	    }
	}
	if {![info exists ${version}(myname)]} {
	    if [catch {PgpMatch_Simple $version [set ${version}Config(myname)] [set ${version}(pubring)]} key] {
		Misc_DisplayText "[string toupper $version] Init" "the name specified in your ${version}-config file\ncouldn't be unambiguously found in your key rings !"
		set ${version}(myname) {}
	    } else {
		set ${version}(myname) [lindex $key 0]
	    }
	}
    } else {
	set ${version}(myname) [lindex [set ${version}(privatekeys)] 0]
    }
}

# should work with pgp5
# not called by gpg support
proc PgpExec_ParseConfigTxt { file configarray } {
    upvar $configarray config

    if [catch {open $file r} in] {
	return
    }

    for {set len [gets $in line]} {$len >= 0} {set len [gets $in line]} {
	if [regexp -nocase "^\[ \t]*(\[a-z]+)\[ \t]*=(\[^#]*)" $line {} option value] {
	    set config([string tolower $option]) [string trim $value " \t\""]
	}
    }
    close $in
}

# added pgp5 support
# added gpg support
proc PgpExec_Encrypt { version in out tokeys } {

    switch $version {
	pgp  { PgpExec_Batch pgp  [concat [list -aet $in -o $out] [Misc_Map key {lindex $key 0} $tokeys]] output }
	pgp5 { PgpExec_Batch pgp5 [concat [list pgpe -at $in -o $out] [foreach id [Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips]] output }
	gpg  { PgpExec_Batch gpg  [concat [list -eat -o $out] [foreach id [Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in] output }
    }

    # pgp thinks he knows better how to name files !
    if {![file exists $out] && [file exists "$out.asc"]} {
	Mh_Rename "$out.asc" $out
    }
    if {![file exists $out]} {
	error "[string toupper $version] refused to generate the encrypted text:\n$output"
    } else {
	return {}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_EncryptSign { version in out sigkey tokeys } {

    switch $version {
	pgp  { PgpExec pgp [concat [list -aset $in -o $out -u [lindex $sigkey 0]] [Misc_Map key {lindex $key 0} $tokeys]] output $sigkey }
	pgp5 { PgpExec pgp5 [concat [list pgpe -ast $in -o $out -u [lindex $sigkey 0]] [foreach id [Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips]] output $sigkey }
	gpg  { PgpExec gpg [concat [list -east -o $out -u [lindex $sigkey 0]] [foreach id [Misc_Map key {lindex $key 0} $tokeys] {lappend recips -r $id}; set recips] $in] output $sigkey }
    }

    # pgp thinks he knows better how to name files !
    if {![file exists $out] && [file exists "$out.asc"]} {
	Mh_Rename "$out.asc" $out
    }
    if {![file exists $out]} {
	error "[string toupper $version] refused to generate the encrypted signed text:\n$output"
    } else {
	return {}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_Sign { version in out sigkey clear } {

    if $clear {
	set clear "+clearsig=on"
	set gpgaction {--clearsign --armor --textmode}
    } else {
	set clear "+clearsig=off"
	set gpgaction {--sign --armor}
    }

    switch $version {
	pgp  { PgpExec pgp  [list $clear -ast $in -u [lindex $sigkey 0] -o $out] output $sigkey }
	pgp5 { PgpExec pgp5 [list pgps $clear -at $in -u [lindex $sigkey 0] -o $out] output $sigkey }
	gpg  { PgpExec gpg  [concat $gpgaction -u [lindex $sigkey 0] -o $out $in] output $sigkey }
    }

    # pgp thinks he knows better how to name files !
    if {![file exists $out] && [file exists "$out.asc"]} {
	Mh_Rename "$out.asc" $out
    }
    if {![file exists $out]} {
    } else {
	return {}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_Decrypt { version in out outvar recipients } {
    global pgp pgp5 gpg pgpPass pgp5Pass gpgPass
    upvar $outvar output

    set recipients [string tolower $recipients]
    set useablekeys [Misc_Filter key {[string first [string tolower [string range [lindex $key 0] 2 end]] $recipients] >= 0} [set ${version}(privatekeys)]]
    set knownkeys [Misc_Filter key {[info exists ${version}Pass([lindex $key 0])]} $useablekeys]

    if {[llength $knownkeys] > 0} {
	set key [lindex $knownkeys 0]
    } elseif {[llength $useablekeys] > 0} {
	set key [lindex $useablekeys 0]
    } else {
	set key {}
    }
    #
    # now call pgp version
    switch $version {
	pgp  { PgpExec pgp  [list $in -o $out] output $key }
	pgp5 { PgpExec pgp5 [list pgpv $in -o $out] output $key }
	gpg  { PgpExec gpg  [list -o $out $in] output $key }
    }
}

# This is called if expectk is enabled.  It seemed the best (easiest
# for me) way to do it was to have this proc terminate when the
# message is finished displaying just as PgpExec_Decrypt would do.
# However, this is a problem for the the expectk script
# (PgpDecryptExpect), which may need to communicate with exmh to ask
# for passwords, etc.  

# My slow and inelegant solution was to tell exmh-bg all the necessary
# information and let PgpDecryptExpect communicate with exmh-bg,
# exiting when done.

# *************** WARNING *****************************
# PgpExec_DecryptExpect does not support PGP 5.0 or GPG
# *************** WARNING *****************************

proc PgpExec_DecryptExpect { version infile outfile msgvar } {
    global exmh pgp getpass exwin pgpPass sedit
    upvar $msgvar msg

    # First update exmh-bg arrays.  I hope that pgp, getpass, 
    # pgpPass, and exwin will be enough.  For exwin seems we have
    # to temporarily change the mtext error to avoid an error when
    # the password window is closed and focus is returned to .msg.t

    send $exmh(bgInterp) [list array set pgp [array get pgp]]
    send $exmh(bgInterp) [list array set getpass [array get getpass]]
    send $exmh(bgInterp) [list array set sedit [array get sedit]]
    send $exmh(bgInterp) [list array set pgpPass [array get pgpPass]]
    send $exmh(bgInterp) [list array set exwin [array get exwin]]
    send $exmh(bgInterp) [list set exwin(mtext) .]

    if [catch {exec $exmh(expectk) -f $exmh(library)/PgpDecryptExpect \
			$infile $outfile $exmh(bgInterp)}] {
	Exmh_Status "Error executing expect process" warn
    }

    set msg [lindex [send $exmh(bgInterp) {list $pgpmsg}] 0]
    send $exmh(bgInterp) [list unset pgpmsg]

    # Now reload pgpPass and exwin from exmh-bg
    array set pgpPass [send $exmh(bgInterp) array get pgpPass]

    # The following appears no longer to be necessary, but now I don't see
    # how to change the position of the getpass window
    #
    #    set exwin(geometry,.getpass) \
    #	   [send $exmh(bgInterp) list {$exwin(geometry,.getpass)}]

    send $exmh(bgInterp) [list unset pgpPass]
}

# added pgp5 support
# added gpg support
proc Pgp_GetPass { version key } {
    global pgp pgp5 gpg pgpPass pgp5Pass gpgPass

    if {[lsearch -glob [set ${version}(privatekeys)] "[lindex $key 0]*"] < 0} {
	return {}
    }

    set keyid [lindex $key 0]
    if [info exists ${version}Pass($keyid)] {
	return [set ${version}Pass($keyid)]
    }
    while 1 {
	if [catch {Misc_GetPass $version "Enter [string toupper $version] password" "password for [lindex $key 1]"} password] {
	    return {}
	} elseif {[PgpExec_CheckPassword $version $password $key]} {
	    if [set ${version}(keeppass)] {
		set ${version}Pass($keyid) $password
		after [expr [set ${version}(passtimeout)] * 60 * 1000] \
			[list Pgp_ClearPassword $version $keyid]
	    }
	    return $password
	}
    }
}

# added pgp5 support
# works with gpg
proc Pgp_ClearPassword { version {keyid {}} } {
    global ${version}Pass
    if {[string length $keyid] == 0} {
	catch {unset ${version}Pass}
	set ${version}Pass() {}
    } else {
	catch {unset ${version}Pass($keyid)}
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_ExtractKeys { version file outvar {interactive 1} } {
    global env
    upvar $outvar output

    set output {}

    switch $version {
	pgp  { set cmd [list PgpExec pgp [list -ka $file] output {} $interactive] }
	pgp5 { set cmd [list PgpExec pgp5 [list pgpk -a $file] output {} $interactive] }
	gpg  { set cmd [list PgpExec gpg [list --import $file] output {} $interactive] }
    }
    if [eval $cmd] {
	Exmh_Status "Key extract failed"
	Exmh_Debug $output
	return 0
    } else {
	Exmh_Debug $output
	return 1
    }
}

# added pgp5 support
# added gpg support
proc PgpExec_GetKeys { version keyid file } {

    switch $version {
	pgp  { set args [list -akx $keyid $file] }
	pgp5 { set args [list pgpk -x $keyid -o $file] }
	gpg  { set args [list --export --armor --textmode -o $file $keyid] }
    }

    if [PgpExec $version $args msg] {
	error $msg
    } else {
	# pgp thinks he knows better how to name files !
	if {![file exists $file] && [file exists "$file.asc"]} {
	    Mh_Rename "$file.asc" $file
	}
	if {![file exists $file]} {
	    error "[string toupper $version] refused to generate the key block for $keyid"
	}
    }
}

# added pgp5 support
# added gpg support
proc Pgp_InterpretOutput { version in outvar } {

    # This function is supposed to take the output given by the other
    # pgp exec procedures and writes different information to the
    # given array.  It is probably best to put all the code that
    # change from PGP version to version in a single place.
    # Currently supported versions: 2.6, 5.0, GPG

    global pgp pgp5 gpg
    upvar $outvar pgpresult

    Exmh_Debug "PGP Output:\n$in"
    regexp {(.*)child process exited abnormally} $in {} in
    set in [string trim $in]

    set pgpresult(ok) 1
    regexp -nocase {key id ([0-9a-f]+)} $in {} pgpresult(keyid)
    # this is for pgp5
    if { ![info exists pgpresult(keyid)] } {
	regexp {0x([0-9A-F]+)} $in {} pgpresult(keyid)
    }
    # and this for gpg
    if { ![info exists pgpresult(keyid)] } {
	regexp {ID ([0-9A-F]+)} $in {} pgpresult(keyid)
    }

  switch $version {
   pgp {
    if [regexp {This.*do not have the secret key.*file.} $in \
	    pgpresult(msg)] {
	set pgpresult(summary) "SecretMissing"
	set pgpresult(ok) 0
    } elseif [regexp {Can't.*can't check signature integrity.*} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "PublicMissing"
	set pgpresult(ok) 1
    } elseif [regexp {Good signature.*} $in pgpresult(msg)] {
	if [regexp {WARNING:.*confidence} $pgpresult(msg)] {
	    set pgpresult(summary) "GoodSignatureUntrusted"
	} else {set pgpresult(summary) "GoodSignatureTrusted"}
    } elseif [regexp {WARNING:.*doesn't match.*} $in \
		  pgpresult(msg)] {
	if [regexp {WARNING:.*confidence.*} $pgpresult(msg)] {
	    set pgpresult(summary) "BadSignatureUntrusted"
	} else {set pgpresult(summary) "BadSignatureTrusted"}
    } elseif [regexp {ERROR} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "UnknownError"
	set pgpresult(msg) $in
	set pgpresult(ok) 0
    } else {
	set pgpresult(summary) "Other"
	set pgpresult(msg) $in
    }
   }
   pgp5 {
    if [regexp {Cannot decrypt.*can only be decrypted by.*} $in \
	    pgpresult(msg)] {
	set pgpresult(summary) "SecretMissing"
	set pgpresult(ok) 0
    } elseif [regexp {Signature by unknown keyid.*} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "PublicMissing"
	set pgpresult(ok) 1
    } elseif [regexp {Good signature.*} $in pgpresult(msg)] {
	if [regexp {WARNING:.*is not trusted.*} $pgpresult(msg)] {
	    set pgpresult(summary) "GoodSignatureUntrusted"
	} else {set pgpresult(summary) "GoodSignatureTrusted"}
    } elseif [regexp {BAD signature.*} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "BadSignature"
    } elseif [regexp {ERROR} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "UnknownError"
	set pgpresult(msg) $in
	set pgpresult(ok) 0
    } else {
	set pgpresult(summary) "Other"
	set pgpresult(msg) $in
    }
    #
    # This is only for beauty
    regsub -all "(\nOpening file \"\[^\"\]*\" type text\\.)|(Opening file \"\[^\"\]*\" type text\\.\n)" \
		$pgpresult(msg) {} pgpresult(msg)
   }
   gpg  {
    if [regexp {decryption failed: +Secret key not available} $in \
	    pgpresult(msg)] {
	set pgpresult(summary) "SecretMissing"
	set pgpresult(ok) 0
    } elseif [regexp {Can't check signature: +Public key not found} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "PublicMissing"
	set pgpresult(ok) 1
    } elseif [regexp {gpg: +Signature made.*Good signature.*} $in pgpresult(msg)] {
	if [regexp {WARNING:.*not.*trusted signature.*} $pgpresult(msg)] {
	    set pgpresult(summary) "GoodSignatureUntrusted"
	} else {set pgpresult(summary) "GoodSignatureTrusted"}
    } elseif [regexp {gpg: +Signature made.*BAD signature.*} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "BadSignature"
    } elseif [regexp {ERROR} $in \
		  pgpresult(msg)] {
	set pgpresult(summary) "UnknownError"
	set pgpresult(msg) $in
	set pgpresult(ok) 0
    } else {
	set pgpresult(summary) "Other"
	set pgpresult(msg) $in
    }
   }
  }

    # DecryptExpect sometimes notifies the user that the
    # file is not encrypted.
    
    if [regexp {Note: File may not have been encrypted} $in] {
	set pgpresult(msg) \
	    "Note: File may not have been encrypted.\n\n$pgpresult(msg)"
    }

    Exmh_Debug OK=$pgpresult(ok) $pgpresult(summary)

    if [set ${version}(shortmsgs)] {
	set pgpresult(msg) [Pgp_ShortenOutput $version $pgpresult(msg) \
				$pgpresult(summary)]
    }
}

# added pgp5 support
proc Pgp_ShortenOutput { version pgpresult summary } {

    switch $version {
	pgp  { regexp {user ("[^"]*")} $pgpresult {} user }
	pgp5 { regexp {by key[^"]*("[^"]*")} $pgpresult {} user }
	gpg  { regexp {from ("[^"]*")} $pgpresult {} user }
    }

    switch $summary {
       SecretMissing {return "Cannot decrypt, missing secret key."}
       PublicMissing {return "Missing public key."}
       GoodSignatureUntrusted {return "Good untrusted signature from $user."}
       GoodSignatureTrusted {return "Good trusted signature from $user."}
       BadSignature {return "Bad signature from $user."}
       BadSignatureTrusted {return "WARNING: Bad trusted signature \
		from $user."}
       BadSignatureUntrusted {return "WARNING: Bad untrusted signature \
		from $user."}
       UnknownError {return "PGP Error while processing message:\n$pgpresult"}
       Other {return $pgpresult}
    }
}

proc Pgp_CheckPoint {  } {
    foreach cmd { PgpMatch_CheckPoint } {
	if {[info command $cmd] != {}} {
	    if [catch {$cmd} err] {
		puts stderr "$cmd: $err"
	    }
	}
    }
}
